use reqwest::Client;
use serde::{Deserialize, Serialize};
use std::env;
use std::error::Error;
use std::fs;
use std::fs::File;
use std::path::Path;

#[derive(Debug, Serialize)]
#[serde(rename_all = "camelCase")]
struct Credentials<'a> {
    login: &'a str,
    password: &'a str,
}

#[derive(Debug, Deserialize)]
#[serde(rename_all = "camelCase")]
struct FeedAttachment {
    #[serde(rename = "_id")]
    id: String,
    content_type: String,
    #[serde(rename = "type")]
    attachment_type: String,
    path: String,
}

#[derive(Debug, Deserialize)]
#[serde(rename_all = "camelCase")]
struct FeedContents {
    body: String,
    attachments: Vec<FeedAttachment>,
}

#[derive(Debug, Deserialize)]
#[serde(rename_all = "camelCase")]
struct FeedItem {
    header_text: String,
    header_subtext: String,
    contents: FeedContents,
}

#[derive(Debug, Deserialize)]
struct Feed {
    #[serde(rename = "_items")]
    items: Vec<FeedItem>,
}

fn fetch_feed(client: &Client) -> Result<Feed, reqwest::Error> {
    client
        .get("https://home.classdojo.com/api/storyFeed?includePrivate=true")
        .send()?
        .json::<Feed>()
}

fn authenticate(
    client: &Client,
    credentials: &Credentials,
) -> Result<reqwest::StatusCode, reqwest::Error> {
    let response = client
        .post("https://home.classdojo.com/api/session")
        .json(&credentials)
        .send()?;
    Ok(response.status())
}

fn fetch_attachment(client: &Client, attachment: &FeedAttachment) -> Result<(), Box<dyn Error>> {
    let raw_path = &format!("out/{}.jpg", attachment.id);
    let path = Path::new(raw_path);
    if path.exists() {
        return Ok(());
    }

    let mut file = File::create(path)?;
    let mut response = client.get(&attachment.path).send()?;
    response.copy_to(&mut file)?;
    Ok(())
}

fn process_feed(client: &Client, feed: Feed) {
    for item in feed.items {
        for attachment in item.contents.attachments {
            if let Err(e) = fetch_attachment(&client, &attachment) {
                println!("Failed to save attachment {}. {}", &attachment.id, e);
                return;
            }
        }
    }
}

fn main() -> Result<(), Box<dyn Error>> {
    let client = Client::builder().cookie_store(true).build()?;
    let credentials = Credentials {
        login: &env::var("CLASSDOJO_LOGIN")?,
        password: &env::var("CLASSDOJO_PASSWORD")?,
    };
    authenticate(&client, &credentials)?;
    let feed = fetch_feed(&client)?;
    fs::create_dir_all("out")?;
    process_feed(&client, feed);

    Ok(())
}
